#extension GL_OES_standard_derivatives : enable
#ifdef GL_ES
precision highp float;
#endif

varying lowp vec2 vTextureCoord;
uniform sampler2D sTexture;
uniform float time;
uniform float paramIntensity;

vec3 getNoise(vec2 coord, float strength, float time);
float rand(vec2 vec);
vec3 tvNoise(vec3 tex, vec2 uv, float strength, float time);
vec3 filter10(vec3 col, vec2 uv, float paramIntensity, float time);
vec3 luma(vec3 col);
vec3 brightnessContrastSaturationMid(vec3 color, float brt, float con, float sat, vec3 mid);
vec3 brightnessContrastSaturation(vec3 color, float brt, float con, float sat);
vec3 filter1(vec3 col, float paramIntensity);

vec3 getNoise(vec2 coord, float strength, float time) {
    float mult = 64.0;
    vec2 i = floor(coord * mult);
    vec2 f = fract(coord * mult);
    float a = rand(i);
    float b = rand(i + vec2(1.0, 0.0));
    float c = rand(i + vec2(0.0, 1.0));
    float d = rand(i + vec2(1.0, 1.0));
    vec2 u = f * f * (3.0 - 2.0 * f);
    float noiseVal = mix(a, b, u.x) +
    (c - a)* u.y * (1.0 - u.x) +
    (d - b) * u.x * u.y;
    return vec3(noiseVal) * strength;
}

float rand(vec2 vec) {
    return fract(sin(dot(vec.xy, vec2(11.7851, 87.433))) * 92834.8274);
}

vec3 tvNoise(vec3 tex, vec2 uv, float strength, float time) {
    float rnd = rand(vec2(time, time));
    vec3 noise = getNoise(vec2(uv.x + rnd, uv.y * 10.0 + rnd), strength * 0.05, time);
    return tex + noise * strength;
}

vec3 filter10(vec3 col, vec2 uv, float paramIntensity, float time) {
    vec3 res = brightnessContrastSaturation(col, 0.7, 0.98, 1.0);
    res = tvNoise(res, uv, paramIntensity, time);
    return mix(col, res, paramIntensity);
}

vec3 luma(vec3 col) {
    float yy = dot(vec3(0.2126729, 0.7151522, 0.0721750), col);
    return vec3(yy);
}

vec3 brightnessContrastSaturationMid(vec3 color, float brt, float con, float sat, vec3 mid) {
    vec3 black = vec3(0.0, 0.0, 0.0);
    float lum = luma(color).x;
    vec3 gray = vec3(lum, lum, lum);
    vec3 brtColor = mix(black, color, brt);
    vec3 conColor = mix(mid, brtColor, con);
    vec3 satColor = mix(gray, conColor, sat);
    return satColor;
}

vec3 brightnessContrastSaturation(vec3 color, float brt, float con, float sat) {
    return brightnessContrastSaturationMid(color, brt, con, sat, vec3(0.5, 0.5, 0.5));
}

vec3 filter1(vec3 col, float paramIntensity) {
    float colorInt = 1.0 + paramIntensity * 0.5;
    float contrastInt = 1.0 + paramIntensity * 0.1;
    return brightnessContrastSaturation(col, colorInt, contrastInt, 1.0);
}

void main() {
    vec2 uv = vTextureCoord;
    vec4 col = texture2D(sTexture, uv);
    col.xyz = filter10(col.xyz, uv, paramIntensity, time);
    gl_FragColor = col;
}