#extension GL_OES_standard_derivatives : enable
#ifdef GL_ES
precision highp float;
#endif

varying lowp vec2 vTextureCoord;
uniform sampler2D sTexture;
uniform sampler2D oTexture;
uniform float time;
uniform float paramIntensity;

vec3 getNoise(vec2 coord, float strength, float time);
float rand(vec2 vec);
vec3 tapeNoise(vec3 tex, vec2 coord, float strength, float time);
vec3 filter11(vec3 col, vec3 col2, vec2 uv, float paramIntensity, float time);
vec4 filter11Preprocess(sampler2D sTexture, vec2 uv, float time);
vec3 luma(vec3 col);
vec3 brightnessContrastSaturationMid(vec3 color, float brt, float con, float sat, vec3 mid);
vec3 brightnessContrastSaturation(vec3 color, float brt, float con, float sat);
vec3 filter1(vec3 col, float paramIntensity);

vec3 getNoise(vec2 coord, float strength, float time) {
    float mult = 64.0;
    vec2 i = floor(coord * mult);
    vec2 f = fract(coord * mult);
    float a = rand(i);
    float b = rand(i + vec2(1.0, 0.0));
    float c = rand(i + vec2(0.0, 1.0));
    float d = rand(i + vec2(1.0, 1.0));
    vec2 u = f * f * (3.0 - 2.0 * f);
    float noiseVal = mix(a, b, u.x) +
    (c - a)* u.y * (1.0 - u.x) +
    (d - b) * u.x * u.y;
    return vec3(noiseVal) * strength;
}

float rand(vec2 vec) {
    return fract(sin(dot(vec.xy, vec2(11.7851, 87.433))) * 92834.8274);
}

vec3 tapeNoise(vec3 tex, vec2 coord, float strength, float time) {
    float rnd = rand(vec2(time, time));
    vec3 noise = getNoise(vec2(coord.x * 0.5 + rnd, coord.y * 40.0 + rnd), strength, time);
    vec3 noiseOP = getNoise(vec2(coord.x * 0.25 + rnd, coord.y * 20.0 + rnd), strength, time * 0.5);
    noise -= noiseOP * 0.6;
    noise = clamp(noise, 0.0, 1.0);
    float noiseLen = length(noise);
    noise = noise * step(0.95, noiseLen);
    float stepVal = smoothstep(0.85, 1.0, 1.0 - coord.y);
    return tex + noise * strength * stepVal;
}

vec3 filter11(vec3 col, vec3 col2, vec2 uv, float paramIntensity, float time) {
    vec3 res = brightnessContrastSaturation(col, 0.9, 1.0, 0.7);
    res = tapeNoise(res, 1.0 - uv, paramIntensity, time);
    res += col2;
    return mix(col, res, paramIntensity);
}

vec4 filter11Preprocess(sampler2D sTexture, vec2 uv, float time) {
    float edge1 = 0.6 + sin(time * 1000.0) * 0.01;
    float edge2 = edge1 + 0.1;
    if (uv.y > edge1 && uv.y < edge2) uv.y = edge1 + smoothstep(0.01, edge2 - edge1, uv.y - edge1) * (edge2 - edge1);
    float bEdge = 0.9 + sin(time * 100.0) * 0.015;
    if (uv.y > bEdge) uv.y = bEdge + (uv.y - bEdge) * 1.2;
    vec4 col0 = texture2D(sTexture, uv);
    vec4 colR = texture2D(sTexture, uv + vec2(0.0, 0.005));
    vec4 colL = texture2D(sTexture, uv - vec2(0.0, 0.005));
    vec4 colT = texture2D(sTexture, uv + vec2(0.005, 0.0));
    vec4 colB = texture2D(sTexture, uv - vec2(0.005, 0.0));
    vec4 col2 = (colR + colL + colT + colB) * 0.25;
    return col0 * 0.75 + col2 * 0.25;
}

vec3 luma(vec3 col) {
    float yy = dot(vec3(0.2126729, 0.7151522, 0.0721750), col);
    return vec3(yy);
}

vec3 brightnessContrastSaturationMid(vec3 color, float brt, float con, float sat, vec3 mid) {
    vec3 black = vec3(0.0, 0.0, 0.0);
    float lum = luma(color).x;
    vec3 gray = vec3(lum, lum, lum);
    vec3 brtColor = mix(black, color, brt);
    vec3 conColor = mix(mid, brtColor, con);
    vec3 satColor = mix(gray, conColor, sat);
    return satColor;
}

vec3 brightnessContrastSaturation(vec3 color, float brt, float con, float sat) {
    return brightnessContrastSaturationMid(color, brt, con, sat, vec3(0.5, 0.5, 0.5));
}

vec3 filter1(vec3 col, float paramIntensity) {
    float colorInt = 1.0 + paramIntensity * 0.5;
    float contrastInt = 1.0 + paramIntensity * 0.1;
    return brightnessContrastSaturation(col, colorInt, contrastInt, 1.0);
}

void main() {
    vec2 uv = vTextureCoord;
    vec4 col = filter11Preprocess(sTexture, uv, time);
    vec4 col2 = texture2D(oTexture, uv);
    col.xyz = filter11(col.xyz, col2.xyz, uv, paramIntensity, time);
    gl_FragColor = col;
}