#extension GL_OES_standard_derivatives : enable
#ifdef GL_ES
precision highp float;
#endif

varying lowp vec2 vTextureCoord;
uniform sampler2D sTexture;
uniform sampler2D oTexture;
uniform float paramIntensity;

vec3 retroSin3(vec3 col, vec2 uv, vec3 col2, vec3 col3, float strenght);
vec3 retroSin(vec3 col, vec2 uv, vec3 col2, float strenght);
vec3 filter12(vec3 col, vec3 col2, vec2 uv, float paramIntensity);
vec3 luma(vec3 col);
vec3 brightnessContrastSaturationMid(vec3 color, float brt, float con, float sat, vec3 mid);
vec3 brightnessContrastSaturation(vec3 color, float brt, float con, float sat);
vec3 filter1(vec3 col, float paramIntensity);

vec3 retroSin3(vec3 col, vec2 uv, vec3 col2, vec3 col3, float strenght) {
    float mi = max(0.3, 0.75 * atan(uv.y));
    float ma = max(0.5, 0.75 * atan(1.0 / (1.0 - uv.x)));
    float val1 = smoothstep(mi, ma, uv.y);
    vec3 fcol = mix(col3, col2, val1);
    return col + vec3(val1) * fcol * strenght;
}

vec3 retroSin(vec3 col, vec2 uv, vec3 col2, float strenght) {
    float sinVal1 = (sin(-0.3 + uv.x * 10.0) + 1.2) * 0.5 * (uv.y * 0.5 + 0.5) * (1.0 - uv.x * 0.8);
    sinVal1 = clamp(sinVal1, 0.0, 0.95);
    return col + vec3(sinVal1) * col2 * strenght;
}

vec3 filter12(vec3 col, vec3 col2, vec2 uv, float paramIntensity) {
    vec3 res = col;
    res = retroSin3(res, uv, vec3(0.7, 0.0, 0.1), vec3(0.1, 0.0, 0.7), paramIntensity);
    return mix(col, res, paramIntensity);
}

vec3 luma(vec3 col) {
    float yy = dot(vec3(0.2126729, 0.7151522, 0.0721750), col);
    return vec3(yy);
}

vec3 brightnessContrastSaturationMid(vec3 color, float brt, float con, float sat, vec3 mid) {
    vec3 black = vec3(0.0, 0.0, 0.0);
    float lum = luma(color).x;
    vec3 gray = vec3(lum, lum, lum);
    vec3 brtColor = mix(black, color, brt);
    vec3 conColor = mix(mid, brtColor, con);
    vec3 satColor = mix(gray, conColor, sat);
    return satColor;
}

vec3 brightnessContrastSaturation(vec3 color, float brt, float con, float sat) {
    return brightnessContrastSaturationMid(color, brt, con, sat, vec3(0.5, 0.5, 0.5));
}

vec3 filter1(vec3 col, float paramIntensity) {
    float colorInt = 1.0 + paramIntensity * 0.5;
    float contrastInt = 1.0 + paramIntensity * 0.1;
    return brightnessContrastSaturation(col, colorInt, contrastInt, 1.0);
}

void main() {
    vec2 uv = vTextureCoord;
    vec4 col = texture2D(sTexture, uv);
    vec4 col2 = texture2D(oTexture, uv);
    col.xyz = filter12(col.xyz, col2.xyz, uv, paramIntensity);
    gl_FragColor = col;
}