#extension GL_OES_standard_derivatives : enable
#ifdef GL_ES
precision highp float;
#endif

varying lowp vec2 vTextureCoord;
uniform sampler2D sTexture;
uniform float paramIntensity;

vec3 luma(vec3 col);
vec3 brightnessContrastSaturationMid(vec3 color, float brt, float con, float sat, vec3 mid);
vec3 brightnessContrastSaturation(vec3 color, float brt, float con, float sat);
vec3 filter3(vec3 col, float paramIntensity);

vec3 luma(vec3 col) {
    float yy = dot(vec3(0.2126729, 0.7151522, 0.0721750), col);
    return vec3(yy);
}

vec3 brightnessContrastSaturationMid(vec3 color, float brt, float con, float sat, vec3 mid) {
    vec3 black = vec3(0.0, 0.0, 0.0);
    float lum = luma(color).x;
    vec3 gray = vec3(lum, lum, lum);
    vec3 brtColor = mix(black, color, brt);
    vec3 conColor = mix(mid, brtColor, con);
    vec3 satColor = mix(gray, conColor, sat);
    return satColor;
}

vec3 brightnessContrastSaturation(vec3 color, float brt, float con, float sat) {
    return brightnessContrastSaturationMid(color, brt, con, sat, vec3(0.5, 0.5, 0.5));
}

vec3 filter3(vec3 col, float paramIntensity) {
    vec3 targetColor = vec3(255.0 / 255.0, 223.0 / 255.0, 196.0 / 255.0);
    vec3 skinColor = vec3(255.0 / 255.0, 170.0 / 255.0, 85.0 / 255.0);
    float prod = dot(col, targetColor);
    vec3 res = col;
    if (col.x < 1.0 && col.y < 1.0 && col.z < 1.0) {
        res = mix(res, skinColor, prod * 0.25);
    }
    res = brightnessContrastSaturation(res, 1.2, 1.0, 1.1);
    return mix(col, res, paramIntensity * 0.5);
}

void main() {
    vec2 uv = vTextureCoord;
    vec4 col = texture2D(sTexture, uv);
    col.xyz = filter3(col.xyz, paramIntensity);
    gl_FragColor = col;
}