#extension GL_OES_standard_derivatives : enable
#ifdef GL_ES
precision highp float;
#endif

varying lowp vec2 vTextureCoord;
uniform sampler2D sTexture;
uniform float paramIntensity;

float processOverlay(float lowerVal, float upperVal);
vec3 colorOverlay(vec3 color, vec3 overlayColor, float grayA);
vec3 filter7(vec3 col, float paramIntensity);

float processOverlay(float lowerVal, float upperVal) {
    float ret = 0.0;
    if (lowerVal >= 0.5) {
        ret = 1.0 - 2.0 * (1.0 - lowerVal) * (1.0 - upperVal);
    } else {
        ret = 2.0 * lowerVal * upperVal;
    }
    return ret;
}

vec3 colorOverlay(vec3 color, vec3 overlayColor, float grayA) {
    color.x = processOverlay(color.x, overlayColor.x);
    color.g = processOverlay(color.y, overlayColor.y);
    color.b = processOverlay(color.z, overlayColor.z);
    float a = grayA;
    float backA = 1.0 - a;
    vec3 gray = vec3(0.21 * color.x + 0.7 * color.y + 0.07 * color.z);
    color.xyz = vec3(backA * color.x + a * gray.x, backA * color.y + a * gray.y, backA * color.z + a * gray.z);
    return color.xyz;
}

vec3 filter7(vec3 col, float paramIntensity) {
    vec3 res = colorOverlay(col, vec3(0.7, 0.5, 0.7), 0.0);
    return mix(col, res, paramIntensity);
}

void main() {
    vec2 uv = vTextureCoord;
    vec4 col = texture2D(sTexture, uv);
    col.xyz = filter7(col.xyz, paramIntensity);
    gl_FragColor = col;
}