#extension GL_OES_standard_derivatives : enable
#ifdef GL_ES
precision highp float;
#endif

varying lowp vec2 vTextureCoord;
uniform sampler2D sTexture;
uniform sampler2D oTexture;
uniform float paramIntensity;

vec3 retroSin2(vec3 col, vec2 uv, vec3 col2, float strenght);
vec3 retroSin(vec3 col, vec2 uv, vec3 col2, float strenght);
vec3 luma(vec3 col);
vec3 brightnessContrastSaturationMid(vec3 color, float brt, float con, float sat, vec3 mid);
vec3 brightnessContrastSaturation(vec3 color, float brt, float con, float sat);
vec3 filter9(vec3 col, vec3 col2, vec2 uv, float paramIntensity);

vec3 retroSin2(vec3 col, vec2 uv, vec3 col2, float strenght) {
    float val1 = (sin(-3.5 + clamp(uv.x * 20.0, 2.0, 8.0)) + 1.2) * 0.9 * (uv.y - 0.45);
    val1 = clamp(val1, 0.0, 0.95);
    float val2 = (sin(1.5 + clamp(uv.x * 10.0, 0.0, 3.0)) + 1.2) * 0.9 * (1.0 - uv.y - 0.35);
    val2 = clamp(val2, 0.0, 0.95);
    float val3 = (sin(-2.0 + clamp(uv.x * 10.0, 6.0, 10.0)) + 1.2) * 0.9 * (uv.y - 0.45);
    val3 = clamp(val3, 0.0, 0.95);
    return col + vec3(val1 + val2 + val3) * col2 * strenght;
}

vec3 retroSin(vec3 col, vec2 uv, vec3 col2, float strenght) {
    float sinVal1 = (sin(-0.3 + uv.x * 10.0) + 1.2) * 0.5 * (uv.y * 0.5 + 0.5) * (1.0 - uv.x * 0.8);
    sinVal1 = clamp(sinVal1, 0.0, 0.95);
    return col + vec3(sinVal1) * col2 * strenght;
}

vec3 luma(vec3 col) {
    float yy = dot(vec3(0.2126729, 0.7151522, 0.0721750), col);
    return vec3(yy);
}

vec3 brightnessContrastSaturationMid(vec3 color, float brt, float con, float sat, vec3 mid) {
    vec3 black = vec3(0.0, 0.0, 0.0);
    float lum = luma(color).x;
    vec3 gray = vec3(lum, lum, lum);
    vec3 brtColor = mix(black, color, brt);
    vec3 conColor = mix(mid, brtColor, con);
    vec3 satColor = mix(gray, conColor, sat);
    return satColor;
}

vec3 brightnessContrastSaturation(vec3 color, float brt, float con, float sat) {
    return brightnessContrastSaturationMid(color, brt, con, sat, vec3(0.5, 0.5, 0.5));
}

vec3 filter9(vec3 col, vec3 col2, vec2 uv, float paramIntensity) {
    vec3 res = brightnessContrastSaturation(col, 0.7, 1.0, 1.0);
    res = step(1.0 - length(col2), 0.5) * res;
    res = retroSin2(res, uv, vec3(0.9, 0.05, 0.0), paramIntensity);
    return mix(col, res, paramIntensity);
}

void main() {
    vec2 uv = vTextureCoord;
    vec4 col = texture2D(sTexture, uv);
    vec4 col2 = texture2D(oTexture, uv);
    col.xyz = filter9(col.xyz, col2.xyz, uv, paramIntensity);
    gl_FragColor = col;
}